﻿using CallCenter.Caches;
using CallCenter.Devices;
using CallCenter.Notifications;
using CallCenter.Repository.SqlSugar;
using CallCenter.Tels;
using CallCenter.Users;
using MediatR;
using Polly.Caching;
using XF.Domain.Cache;

namespace CallCenter.Application.Handlers
{
    public class OnlineNotificationHandler : INotificationHandler<OnlineNotification>
    {
        private readonly ITelRepository _telRepository;
        private readonly ITelCacheManager _telCacheManager;
        private readonly ITypedCache<Tel> _typedCache;
        private readonly IUserCacheManager _userCacheManager;
        private readonly IUserRepository _userRepository;
        private readonly IDeviceManager _deviceManager;
        public OnlineNotificationHandler(ITelRepository telRepository, ITelCacheManager telCacheManager, ITypedCache<Tel> typedCache, IUserCacheManager userCacheManager, IUserRepository userRepository, IDeviceManager deviceManager)
        {
            _telRepository = telRepository;
            _telCacheManager = telCacheManager;
            _typedCache = typedCache;
            _userCacheManager = userCacheManager;
            _userRepository = userRepository;
            _deviceManager = deviceManager;
        }

        /// <summary>Handles a notification</summary>
        /// <param name="notification">The notification</param>
        /// <param name="cancellationToken">Cancellation token</param>
        public async Task Handle(OnlineNotification notification, CancellationToken cancellationToken)
        {
            var telModel = _telCacheManager.GetTel(notification.TelNo);
            telModel.TelStatus = ETelStatus.Ready;
            telModel.RegisterIP = notification.RegisterIP;
            await _telRepository.UpdateAsync(telModel,cancellationToken);
            _typedCache.Set(notification.TelNo, telModel);

        }
    }
}
